@@ -1,23 +1,27 @@
 `H2fun` <-
-function(web){
+function(web, H2_integer=TRUE){
     # function to calculate a measure of specialisation in a bipartite web: H2'
     #
     # web   a bipartite interaction web, with e.g. pollinators as columns and plants as rows
     #
     # returns the normalised H2' and its subcomponents H2max, H2min and H2uncorrected
     #
-    # by Jochen Frnd, Uni Wrzburg (some streamlining by Carsten Dormann, UFZ Leipzig); April 2007
-    # This function is based on the paper by Blthgen, Menzel & Blthgen 2006 (BMC Ecology).
+    # by Jochen Fr¸nd, Uni W¸rzburg (some streamlining by Carsten Dormann, UFZ Leipzig); April 2007
+    # This function is based on the paper by Bl¸thgen, Menzel & Bl¸thgen 2006 (BMC Ecology).
     # for web implementation see also: http://itb1.biologie.hu-berlin.de/~nils/stat/
     #
     # Example:
     # set.seed(8793)
     # web <- matrix(rpois(81, 0.5), 9)
     # H2prime(web)
+    
+    # code proposals by Jochen to deal with non-integer data:
+    if (H2_integer & any((web %% 1) != 0)) stop("web does not contain integers! maybe you should set H2_integer to FALSE")
+    
 
-    tot <- sum(web)       #smtliche Interaktionen im Netz
-    rs <- rowSums(web)    #Interaktion der Pflanze mit smtlichen Bestubern
-    cs <- colSums(web)    #smtliche Interaktionen des jeweiligen Bestbers
+    tot <- sum(web)       #s‰mtliche Interaktionen im Netz
+    rs <- rowSums(web)    #Interaktion der Pflanze mit s‰mtlichen Best‰ubern
+    cs <- colSums(web)    #s‰mtliche Interaktionen des jeweiligen Best‰bers
 
    #--------------- H2 uncorrected------------
     H2uncorr = -sum(web/tot*log(web/tot), na.rm=TRUE)
@@ -26,21 +30,24 @@ function(web){
     # Key idea here is to allocate interactions one-by-one into the places where
     # they fit best according to the non-integer optimal web. Easy! (Not.)
     exexpec <- outer(rs, cs/tot) # non-integer optimal web
-    expec <- matrix(0, nrow(web), ncol(web)) # empty web
-    difexp <- exexpec-expec # where are differences between non-integer and 0-web greatest?
-    newweb <- floor(exexpec)  # start new web
-    webfull <- matrix("no", nrow(web), ncol(web)) # makes boolean web, set to 0
-    while (sum(newweb) < tot) {
-       webfull[which(rowSums(newweb)==rs),] <- "yo" # sets columns/rows with correct cs/rs to 1
-       webfull[,which(colSums(newweb)==cs)] <- "yo"
-       OK <- webfull=="no" # matrix of potential cells
-       smallestpossible <- newweb==min(newweb[OK])  # find cell with lowest number of interactions (e.g. 0)
-       greatestdif <- max(difexp[smallestpossible & OK]) # find cell value with largest different between "is" and "should"
-       bestone <- which(OK & smallestpossible & difexp==greatestdif ) # find cell for all three conditions
-       if (length(bestone)>1) bestone <- sample(bestone,1) # select randomly a cell, if different are possible
-       newweb[bestone] <- newweb[bestone] + 1 # put an interaction into that cell
-       difexp <- exexpec - newweb
-    }
+    if (!H2_integer) {
+    	newweb <- exexpec
+    } else {
+    	expec <- matrix(0, nrow(web), ncol(web)) # empty web
+    	difexp <- exexpec-expec # where are differences between non-integer and 0-web greatest?
+    	newweb <- floor(exexpec)  # start new web
+    	webfull <- matrix("no", nrow(web), ncol(web)) # makes boolean web, set to 0
+    	while (sum(newweb) < tot) {
+       	webfull[which(rowSums(newweb)==rs),] <- "yo" # sets columns/rows with correct cs/rs to 1
+       	webfull[,which(colSums(newweb)==cs)] <- "yo"
+       	OK <- webfull=="no" # matrix of potential cells
+	       smallestpossible <- newweb==min(newweb[OK])  # find cell with lowest number of interactions (e.g. 0)
+       	greatestdif <- max(difexp[smallestpossible & OK]) # find cell value with largest different between "is" and "should"
+       	bestone <- which(OK & smallestpossible & difexp==greatestdif ) # find cell for all three conditions
+       	if (length(bestone)>1) bestone <- sample(bestone,1) # select randomly a cell, if different are possible
+       	newweb[bestone] <- newweb[bestone] + 1 # put an interaction into that cell
+       	difexp <- exexpec - newweb
+    	}
         
 #    ## what may happen is that their are very few too-high-entries, but many too-low-entries!!
 #    difweb <- newweb - exexpec #positive values indicate too-high-entries
@@ -61,23 +68,23 @@ function(web){
 
     
 #  # Hmaxfind     IMPROVED CODE FROM JOCHEN ------------------------------------
-  if (max(exexpec)>0.3679*tot) {    # 0.3679 is the proportion yielding maximal contribution
+  	if (max(exexpec)>0.3679*tot) {    # 0.3679 is the proportion yielding maximal contribution
     # warning("one cell dominates too extremely, H2max can probably not be estimated correctly")
     # further modification to match expected values even better... ; great advance, but can get caught!!  introduce random step if this happens
-    for (tries in 1:500) {# reduces overfits, but NOT underfits!!
-      newmx <- newweb                         # "hin- und herschieben"
-      difexp <- exexpec - newmx #newmx=newweb!
-      greatestdif <- difexp==min(difexp)
-      if (length(which(greatestdif))>1) {
-          largestvalue = newmx==max(newmx[greatestdif])  # evtl den grten auswhlen
-          first <- greatestdif & largestvalue
-      } else {first=greatestdif}   # "first" is a boolean matrix
-      newmx[first][1] <-  newmx[first][1] - 1                                    # remove one interaction from one cell (with largest difference to expected values; "too large")
-      throw = which(rowSums(first)>0)[1] ;  thcol = which(colSums(first)>0)[1]  # find row- and column-number of removed cell (not elegant!!)
-      mr=max(difexp[throw,])   ; mc=max(difexp[,thcol])                         # find largest difference value in row and column to expected; "too small"
-      if (mr>=mc) {scnd = which(difexp[throw,]==mr) [1]                         # reallocation; start in row
-                   newmx[throw,scnd] = newmx[throw,scnd]+1                      # put in cell
-                   thrd=which(difexp[,scnd]==min(difexp[,scnd]))[1]
+    	for (tries in 1:500) {# reduces overfits, but NOT underfits!!
+      		newmx <- newweb                         # "hin- und herschieben"
+      		difexp <- exexpec - newmx #newmx=newweb!
+      		greatestdif <- difexp==min(difexp)
+      		if (length(which(greatestdif))>1) {
+          		largestvalue = newmx==max(newmx[greatestdif])  # evtl den grˆﬂten ausw‰hlen
+          		first <- greatestdif & largestvalue
+      		} else {first=greatestdif}   # "first" is a boolean matrix
+      		newmx[first][1] <-  newmx[first][1] - 1                                    # remove one interaction from one cell (with largest difference to expected values; "too large")
+      		throw = which(rowSums(first)>0)[1] ;  thcol = which(colSums(first)>0)[1]  # find row- and column-number of removed cell (not elegant!!)
+      		mr=max(difexp[throw,])   ; mc=max(difexp[,thcol])                         # find largest difference value in row and column to expected; "too small"
+      		if (mr>=mc) {scnd = which(difexp[throw,]==mr) [1]                         # reallocation; start in row
+              	 newmx[throw,scnd] = newmx[throw,scnd]+1                      # put in cell
+                  thrd=which(difexp[,scnd]==min(difexp[,scnd]))[1]
                    newmx[thrd,scnd] = newmx[thrd,scnd] - 1                      # remove interaction from "too large cell" in that column
                    newmx[thrd,thcol] = newmx[thrd,thcol] + 1                    # put interaction in the cell that recovers original r/c-sums
                   } else {                                                      # as above, but first reallocation in column
@@ -87,11 +94,11 @@ function(web){
                    newmx[scnd,thrd] = newmx[scnd,thrd] - 1
                    newmx[throw,thrd] = newmx[throw,thrd] + 1
                   }
-      }
-      newweb <- newmx
-  }     # end Hmaxfind
-#  Hmax=H2(newmx)    
-
+      		}
+      		newweb <- newmx
+  		}     # end Hmaxfind
+		#  Hmax=H2(newmx)    
+	}
     H2_max <- -sum(newweb/tot*log(newweb/tot), na.rm=TRUE)   
    
     
@@ -127,4 +134,4 @@ function(web){
 
 
 # This is still a bit unsatisfactory: First, we build a web which contains too many too-high-values, then we correct it. Ideally, the check for BOTH, too-high and too-low values would be done in the building of the web. That would certainly save computational time!!
-# Currently, the build-up is relatively complicated and slow because marginal totals need to be observed.
\ No newline at end of file
+# Currently, the build-up is relatively complicated and slow because marginal totals need to be observed.    
\ No newline at end of file
